﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T ship)
        {
            // TODO вставка в начало набора
            if (_places[Count - 1] != null)
                return -1;
            return Insert(ship,0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T ship, int position)
        {
            // TODO проверка позиции
            // TODO проверка, что элемент массива по этой позиции пустой,  если нет, то
            // проверка, что после вставляемого элемента в массиве есть пустой элемент
            // сдвиг всех объектов, находящихся справа от позиции до первого пустого элемента
            // TODO вставка по позиции_places[position] = car;
            if (!(position >= 0 && position < Count))
                return -1;
            if (_places[position] != null)
            {
                int ind = position;
                while (ind < Count && _places[ind] != null)
                    ind++;
                if (ind == Count)
                    return -1;
                for (int i = ind - 1; i >= position; i--)
                    _places[i + 1] = _places[i];
            }
            _places[position] = ship;
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            // TODO проверка позиции
            // TODO удаление объекта из массива, присвоив элементу массива значение null
            if (!(position >= 0 && position < Count) || _places[position] == null)
                return false;
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];

            }
            set
            {
                if(!(position >= 0 && position < Count && _places.Count < _maxCount))
                {
                    return;
                }
                _places.Insert(position, value);
                return;
            }

        }
        public IEnumerable<T> GetShips(int? maxShips = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxShips.HasValue && i == maxShips.Value)
                {
                    yield break;
                }
            }
        }
    }
}

